#ifndef OP5_LICENSE_ERROR_H_
#define OP5_LICENSE_ERROR_H_

enum OP5LIC_STATUS {
	OP5LIC_STATUS_OK,
	OP5LIC_STATUS_NOT_EXIST,
	OP5LIC_STATUS_UNSIGNED,
	OP5LIC_STATUS_INVALID_SIGNATURE,
	OP5LIC_STATUS_DECODING_ERROR,
	OP5LIC_STATUS_NSS_FAILURE,
	OP5LIC_STATUS_LUA_ERRSYNTAX,
	OP5LIC_STATUS_LUA_ERRMEM,
	OP5LIC_STATUS_LUA_ERRERR,
	OP5LIC_STATUS_LUA_ERRRUN,
	OP5LIC_STATUS_INVALID_ENTRYPOINT,
	OP5LIC_STATUS_MISSING_METADATA,
	OP5LIC_STATUS_BACKBOX_INVALID
};

typedef enum OP5LIC_STATUS LicenseStatus;
struct op5lic_error_t {
	LicenseStatus status;
	char * description; /*Error as intended to be presented to end-user*/
	char * file;
	unsigned int lineno;
};

/*
 * Creates and returns a new op5lic_error_t. Note that the returned pointer is reusable.
 */
struct op5lic_error_t * op5lic_error_create(void);

 /*
  * Destroys a op5lic_error_t as returned from op5lic_error_create().
  */
void op5lic_error_destroy(struct op5lic_error_t * err);

/*
 * Returns a string describing the supplied status code
 * @return A descriptive message of the status code contained in `err'
 */
const char *op5lic_strerror(struct op5lic_error_t *err);

/*
 * Maps a given Lua error code to the corresponding LicenseStatus
 * value
 * @param lua_error The lua status code
 * @return The corresponding LicenseStatus status code
 */
LicenseStatus op5lic_error_from_luastatus(int lua_error);

/*
 * Helper function that fills in an op5lic_error_t.
 */
void _op5lic_error_set(struct op5lic_error_t * err, LicenseStatus status, const char * description, const char *file, unsigned int lineno);
#define op5lic_error_set(Err, Status, Description) do{ if (err != NULL) {_op5lic_error_set(Err, Status, Description, __FILE__, __LINE__); } } while(0)
#endif
